
=head1 NAME

SuikaWiki::DB::FileSystem::SuikaWikiMetaInfo09 --- SuikaWiki WikiDatabase: SuikaWikiMetaInfo/0.9 one-file meta properties database

=head1 DESCRIPTION

This module provides SuikaWiki WikiDatabase interface to SuikaWikiMetaInfo/0.9
format of database.  It can keep keys with multiple properties into
one file.  It is originally implemented as supplemental database of
Yuki::YukiWikiDBMeta.

This module is part of SuikaWiki.

=cut

package SuikaWiki::DB::FileSystem::SuikaWikiMetaInfo09;
use strict;
our $VERSION=do{my @r=(q$Revision: 1.8 $=~/\d+/g);sprintf "%d."."%02d" x $#r,@r};
require SuikaWiki::DB::Util;
push our @ISA, 'SuikaWiki::DB::Util::template';
my $Sep = '//';
my $Self = '.';
my $Parent = '..';

sub new ($%) {
  my $self = shift->SUPER::new (@_);
  my %opt = @_;
  $self->{directory} = $opt{directory}
    or report SuikaWiki::DB::Util::Error
         -type => 'DB_METHOD_PARAMETER_REQUIRED',
         -object => $self, method => 'new',
         parameter => 'directory';
  $self->{directory} .= '/' unless substr ($self->{directory}, -1, 1) eq '/';
  $self->{prefix} = $opt{prefix} || 'mt--';
  $self->{suffix} = $opt{suffix} || '.dat';
  $self;
}

sub __encode_base16 ($$) {
  my ($self, $s) = @_;
  $s =~ s/(.)/sprintf '%02X', ord $1/ges;
  $s;
}
sub __decode_base16 ($$) {
  my ($self, $s) = @_;
  $s =~ s/([0-9A-Fa-f]{2})/chr hex $1/ge;
  $s;
}

sub ___open_prop ($$) {
  my ($self, $opt) = @_;
  local $Error::Depth = $Error::Depth + 1;
  my $file = $self->{directory}
            .$self->{prefix}
            .$self->__encode_base16 ($opt->{prop})
            .$self->{suffix};
  unless (-e $file) {
    $self->{db_hash}->{$opt->{prop}} = {};
  } else {
    CORE::open DB, '<', $file 
            or report SuikaWiki::DB::Util::Error
                      -type => 'STOP_DB_PROP_CANT_OPEN', file => $file,
                      msg => $!, prop => $opt->{prop},
                      -object => $self, method => '__read_meta';
    binmode DB;
    local $/ = undef;
    my $val = <DB>;
    CORE::close DB;
    
    if ($val =~ s!^\#\?SuikaWikiMetaInfo/0.9[^\x02]*\x02!!s) {
      $self->{db_hash}->{$opt->{prop}} = {map {split /\x1F/, $_, 2}
                                          split /\x1E/, $val};
    } elsif (length $val) {
      report SuikaWiki::DB::Util::Error
        -type => 'DB_UNSUPPORTED_FORMAT', file => $file,
        method => '__read_meta', -object => $self,
        prop => $opt->{prop};
    } else {  ## Blank file
      $self->{db_hash}->{$opt->{prop}} = {};
    }
  }
  $self->{prop_modified}->{$opt->{prop}} = 0;
}

sub ___close_prop ($$) {
  my ($self, $opt) = @_;
  local $Error::Depth = $Error::Depth + 1;
  unless ($self->{prop_modified}->{$opt->{prop}}) {
    report SuikaWiki::DB::Util::Error
      -type => 'INFO_DB_PROP_NOT_MODIFIED',
      -object => $self, method => '___close_prop',
      prop => $opt->{prop};
    return;
  }
  if ($self->{lock} and not $self->{lock}->locked) {
    $self->{lock}->lock
      or report SuikaWiki::DB::Util::Error
           -type => 'LOCK_START', method => '__write_meta',
           -object => $self, prop => $opt->{prop};
  }
  report SuikaWiki::DB::Util::Error 
    -type => 'DB_READ_ONLY', method => '___close_prop'
    -object => $self,
    if $self->{lock} and not $self->{lock}->writable;
  my $file = $opt->{prop};
  $file = $self->{directory}.$self->{prefix}.$self->__encode_base16 ($file)
         .$self->{suffix};
  
  my $temp = $file.'.'.time.'.tmp';
  CORE::open FILE, '>', $temp
    or report SuikaWiki::DB::Util::Error
         -type => 'STOP_DB_PROP_CANT_SAVE', msg => $!,
         method => '___close_prop', file => $file,
         prop => $opt->{prop}, -object => $self;
  binmode FILE;
  print FILE "#?SuikaWikiMetaInfo/0.9\n\x02"
            .join "\x1E",
             map {
               my ($n, $v) = ($_, $self->{db_hash}->{$opt->{prop}}->{$_});
               for ($n, $v) {s/([\x02\x1C-\x1F])/sprintf '\\x%02X', ord $1/ge}
               $n."\x1F".$v;
             }
             grep {length $self->{db_hash}->{$opt->{prop}}->{$_}}
             keys %{$self->{db_hash}->{$opt->{prop}}};
  CORE::close FILE;
  if (-e $file) {
    unlink $file or report SuikaWiki::DB::Util::Error
                      -type => 'STOP_DB_PROP_CANT_SAVE', file => $file,
                      method => '___close_prop (unlink)',
                      msg => $!, prop => $opt->{prop},
                      -object => $self;
  }
  rename $temp => $file
    or report SuikaWiki::DB::Util::Error
         -type => 'STOP_DB_PROP_CANT_SAVE', file => $file,
         method => '___close_prop (rename)',
         msg => $!, prop => $opt->{prop},
         -object => $self;
  $self->{prop_modified}->{$opt->{prop}} = 0;
  report SuikaWiki::DB::Util::Error
    -type => 'INFO_DB_PROP_WROTE',
    -object => $self, method => '___close_prop',
    file => $file, prop => $opt->{prop};
  CORE::delete $self->{db_hash}->{$opt->{prop}};
}

sub __name2name ($$) {
  my ($self, $key) = @_;
  join $Sep, @$key;
}

sub __check_name ($$) {
  my ($self, $key) = @_;
  return 0 unless ref $key;
  for (@$key) {
    return 0 unless $_;  # '' or 0 or undef
    return 0 if index ($_, $Sep) > -1;
    return 0 if $_ eq $Self || $_ eq $Parent;
  }
  return 1;  
}

sub get ($$$) {
  my ($self, $prop, $key) = @_;
  unless ($self->{opened}->{$prop}) {
    local $Error::Depth = $Error::Depth + 1;
    $self->open_prop (prop => $prop);
  }
  $self->{db_hash}->{$prop}->{ $self->__name2name ($key) };
}

sub set ($$$$) {
  my ($self, $prop, $key => $value) = @_;
  report SuikaWiki::DB::Util::Error 
    -type => 'KEY_INVALID_NAME',
    -object => $self, method => 'set',
    prop => $prop, key => $key
    unless $self->__check_name ($key);
  unless ($self->{opened}->{$prop}) {
    local $Error::Depth = $Error::Depth + 1;
    $self->open_prop (prop => $prop);
  }
  $self->{prop_modified}->{$prop} = 1;
  $self->{db_hash}->{$prop}->{ $self->__name2name ($key) } = $value;
}

sub exist ($$$) {
  my ($self, $prop, $key) = @_;
  return 0 unless $self->__check_name ($key);
  unless ($self->{opened}->{$prop}) {
    local $Error::Depth = $Error::Depth + 1;
    $self->open_prop (prop => $prop);
  }
  CORE::exists $self->{db_hash}->{$prop}->{ $self->__name2name ($key) };
}

sub delete ($$$) {
  my ($self, $prop, $key) = @_;
  unless ($self->{opened}->{$prop}) {
    local $Error::Depth = $Error::Depth + 1;
    $self->open_prop (prop => $prop);
  }
  $self->{prop_modified}->{$prop} = 1;
  CORE::delete $self->{db_hash}->{$prop}->{ $self->__name2name ($key) };
}

sub keys ($$;%) {
  my ($self, $prop, %opt) = @_;
  unless ($self->{opened}->{$prop}) {
    local $Error::Depth = $Error::Depth + 1;
    $self->open_prop (prop => $prop);
  }
  my $ns = $self->__name2name ($opt{-ns});
  $ns .= $Sep if length $ns;
  my $ns_l = length $ns;
  my @result;
  for (CORE::keys %{$self->{db_hash}->{$prop}}) {
    ## Namespace matchs
    if (substr ($_, 0, $ns_l) eq $ns) {
      ## Child or grandchild in recursive mode
      if ($opt{-recursive} || not (index ($_, $Sep) > $ns_l - 1)) {
        push @result, $_;
      }
    }
  }
  return map {[split /\Q$Sep\E/, $_]} @result;
}

# close: Inherited

# DESTROY: Inherited

=head1 METHODS

This module provides common interface of SuikaWiki WikiDatabase
modules. See C<SuikaWiki::DB>.

=head1 AUTHOR

Wakaba <w@suika.fam.cx>.

=head1 LICENSE

Copyright 2003-2004 Wakaba <w@suika.fam.cx>.  All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

1; # $Date: 2004/04/01 04:45:16 $
